import { WebGroupState } from './index.common.doc';
import { webChannelDefaultOptions } from './WebChannel';
import { wcs, WebGroup } from './WebChannelFacade';
import { Route, WebSocketBuilder } from './WebSocketBuilder';
var urlLib = require('url');
var uws = require('uws');
var botDefaultOptions = {
    url: '',
    perMessageDeflate: false,
    leaveOnceAlone: true,
    server: undefined,
    webGroupOptions: webChannelDefaultOptions,
};
var Bot = (function () {
    function Bot(options) {
        this.wcOptions = Object.assign({}, webChannelDefaultOptions, options.webGroupOptions);
        var fullOptions = Object.assign({}, botDefaultOptions, options);
        fullOptions.webGroupOptions = this.wcOptions;
        this.leaveOnceAlone = fullOptions.leaveOnceAlone;
        this.server = fullOptions.server;
        this.listenUrl = fullOptions.url;
        this.perMessageDeflate = fullOptions.perMessageDeflate;
        this.webGroups = new Map();
        this.onWebGroup = function none() { };
        this.onError = function none() { };
        this.init();
    }
    Object.defineProperty(Bot.prototype, "url", {
        get: function () {
            if (this.listenUrl !== '') {
                return this.listenUrl;
            }
            else {
                var info = this.server.address();
                return "ws://" + info.address + ":" + info.port;
            }
        },
        enumerable: true,
        configurable: true
    });
    Bot.prototype.init = function () {
        var _this = this;
        this.webSocketServer = new uws.Server({
            perMessageDeflate: this.perMessageDeflate,
            verifyClient: function (info) { return _this.validateConnection(info); },
            server: this.server,
        });
        var serverListening = this.server || this.webSocketServer;
        serverListening.on('listening', function () { return WebSocketBuilder.listenUrl.next(_this.url); });
        this.webSocketServer.on('error', function (err) {
            WebSocketBuilder.listenUrl.next('');
            _this.onError(err);
        });
        this.webSocketServer.on('connection', function (ws) {
            var _a = _this.readUrl(ws.upgradeReq.url), route = _a.route, wcId = _a.wcId, senderId = _a.senderId, key = _a.key;
            switch (route) {
                case Route.INTERNAL: {
                    var wg = _this.webGroups.get(wcId);
                    var wc = wcs.get(wg);
                    wc.webSocketBuilder.newInternalWebSocket(ws, senderId);
                    break;
                }
                case Route.JOIN: {
                    var wg = _this.webGroups.get(wcId);
                    var wc = wcs.get(wg);
                    wc.webSocketBuilder.newJoinWebSocket(ws);
                    break;
                }
                case Route.INVITE: {
                    var wg = new WebGroup(_this.wcOptions);
                    _this.webGroups.set(wcId, wg);
                    var wc_1 = wcs.get(wg);
                    if (_this.leaveOnceAlone) {
                        wc_1.onAlone = function () {
                            wc_1.leave();
                            _this.webGroups.delete(wcId);
                        };
                    }
                    wc_1.init(key, wcId);
                    _this.onWebGroup(wg);
                    wc_1.webSocketBuilder.newInviteWebSocket(ws);
                    break;
                }
            }
        });
    };
    Bot.prototype.validateConnection = function (info) {
        var _a = this.readUrl(info.req.url), route = _a.route, wcId = _a.wcId, senderId = _a.senderId, key = _a.key;
        if (wcId === undefined) {
            return false;
        }
        switch (route) {
            case Route.INTERNAL:
                return this.webGroups.has(wcId) && !!senderId;
            case Route.INVITE: {
                var wg = this.webGroups.get(wcId);
                return !!key && (wg === undefined || wg.state === WebGroupState.LEFT);
            }
            case Route.JOIN: {
                var wg = this.webGroups.get(wcId);
                return !!key && wg !== undefined && wg.key === key && wg.state !== WebGroupState.LEFT;
            }
            default:
                return false;
        }
    };
    Bot.prototype.readUrl = function (url) {
        var _a = urlLib.parse(url, true), pathname = _a.pathname, _b = _a.query, senderId = _b.senderId, wcId = _b.wcId, key = _b.key;
        var lastRoute = pathname;
        if (pathname.endsWith('/')) {
            lastRoute = pathname.substr(0, pathname.length - 1);
        }
        lastRoute = lastRoute.split('/').pop();
        return {
            route: lastRoute,
            wcId: wcId ? Number(wcId) : undefined,
            senderId: senderId ? Number(senderId) : undefined,
            key: key,
        };
    };
    return Bot;
}());
export { Bot };
//# sourceMappingURL=Bot.js.map