import { WebChannel } from './WebChannel';
export var wcs = new WeakMap();
var WebGroup = (function () {
    function WebGroup(options) {
        if (options === void 0) { options = {}; }
        var wc = new WebChannel(options);
        wcs.set(this, wc);
        this.id = undefined;
        Reflect.defineProperty(this, 'id', { configurable: false, enumerable: true, get: function () { return wc.id; } });
        this.myId = undefined;
        Reflect.defineProperty(this, 'myId', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.myId; },
        });
        this.key = undefined;
        Reflect.defineProperty(this, 'key', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.key; },
        });
        this.members = undefined;
        Reflect.defineProperty(this, 'members', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.members; },
        });
        this.topology = undefined;
        Reflect.defineProperty(this, 'topology', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.topologyEnum; },
        });
        this.state = undefined;
        Reflect.defineProperty(this, 'state', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.state; },
        });
        this.signalingState = undefined;
        Reflect.defineProperty(this, 'signalingState', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.signaling.state; },
        });
        this.signalingServer = undefined;
        Reflect.defineProperty(this, 'signalingServer', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.signaling.url; },
        });
        this.autoRejoin = undefined;
        Reflect.defineProperty(this, 'autoRejoin', {
            configurable: false,
            enumerable: true,
            get: function () { return wc.autoRejoin; },
            set: function (value) { return (wc.autoRejoin = value); },
        });
        this.onMessage = undefined;
        Reflect.defineProperty(this, 'onMessage', {
            configurable: true,
            enumerable: true,
            get: function () { return (wc.onMessage.name === 'none' ? undefined : wc.onMessage); },
            set: function (handler) {
                if (typeof handler !== 'function') {
                    wc.onMessage = function none() { };
                }
                else {
                    wc.onMessage = handler;
                }
            },
        });
        this.onMemberJoin = undefined;
        Reflect.defineProperty(this, 'onMemberJoin', {
            configurable: true,
            enumerable: true,
            get: function () { return (wc.onMemberJoin.name === 'none' ? undefined : wc.onMemberJoin); },
            set: function (handler) {
                if (typeof handler !== 'function') {
                    wc.onMemberJoin = function none() { };
                }
                else {
                    wc.onMemberJoin = handler;
                }
            },
        });
        this.onMemberLeave = undefined;
        Reflect.defineProperty(this, 'onMemberLeave', {
            configurable: true,
            enumerable: true,
            get: function () { return (wc.onMemberLeave.name === 'none' ? undefined : wc.onMemberLeave); },
            set: function (handler) {
                if (typeof handler !== 'function') {
                    wc.onMemberLeave = function none() { };
                }
                else {
                    wc.onMemberLeave = handler;
                }
            },
        });
        this.onStateChange = undefined;
        Reflect.defineProperty(this, 'onStateChange', {
            configurable: true,
            enumerable: true,
            get: function () { return (wc.onStateChange.name === 'none' ? undefined : wc.onStateChange); },
            set: function (handler) {
                if (typeof handler !== 'function') {
                    wc.onStateChange = function none() { };
                }
                else {
                    wc.onStateChange = handler;
                }
            },
        });
        this.onSignalingStateChange = undefined;
        Reflect.defineProperty(this, 'onSignalingStateChange', {
            configurable: true,
            enumerable: true,
            get: function () {
                return wc.onSignalingStateChange.name === 'none' ? undefined : wc.onSignalingStateChange;
            },
            set: function (handler) {
                if (typeof handler !== 'function') {
                    wc.onSignalingStateChange = function none() { };
                }
                else {
                    wc.onSignalingStateChange = handler;
                }
            },
        });
    }
    WebGroup.prototype.join = function (key) {
        var wc = wcs.get(this);
        if (wc) {
            return wc.join(key);
        }
        throw new Error('WebChannel is undefined');
    };
    WebGroup.prototype.invite = function (url) {
        var wc = wcs.get(this);
        if (wc) {
            return wc.invite(url);
        }
        throw new Error('WebChannel is undefined');
    };
    WebGroup.prototype.leave = function () {
        var wc = wcs.get(this);
        if (wc) {
            return wc.leave();
        }
        throw new Error('WebChannel is undefined');
    };
    WebGroup.prototype.send = function (data) {
        var wc = wcs.get(this);
        if (wc) {
            return wc.send(data);
        }
        throw new Error('WebChannel is undefined');
    };
    WebGroup.prototype.sendTo = function (id, data) {
        var wc = wcs.get(this);
        if (wc) {
            return wc.sendTo(id, data);
        }
        throw new Error('WebChannel is undefined');
    };
    return WebGroup;
}());
export { WebGroup };
//# sourceMappingURL=WebChannelFacade.js.map