import { merge } from 'rxjs';
import { filter, map } from 'rxjs/operators';
var Service = (function () {
    function Service(serviceId, proto) {
        this.serviceId = serviceId;
        this.proto = proto;
    }
    Service.prototype.useWebChannelStream = function (wc) {
        var _this = this;
        return {
            id: wc.STREAM_ID,
            message: wc.messageFromStream.pipe(filter(function (_a) {
                var serviceId = _a.serviceId;
                return serviceId === _this.serviceId;
            }), map(function (_a) {
                var channel = _a.channel, senderId = _a.senderId, recipientId = _a.recipientId, content = _a.content;
                return ({
                    channel: channel,
                    senderId: senderId,
                    recipientId: recipientId,
                    msg: _this.decode(content),
                });
            })),
            send: function (content, id) {
                wc.sendOverStream({
                    senderId: wc.myId,
                    recipientId: id,
                    serviceId: _this.serviceId,
                    content: content instanceof Uint8Array ? content : _this.encode(content),
                });
            },
        };
    };
    Service.prototype.useSignalingStream = function (sig) {
        var _this = this;
        return {
            id: sig.STREAM_ID,
            message: sig.messageFromStream.pipe(filter(function (_a) {
                var serviceId = _a.serviceId;
                return serviceId === _this.serviceId;
            }), map(function (_a) {
                var senderId = _a.senderId, content = _a.content;
                return ({ senderId: senderId, msg: _this.decode(content) });
            })),
            send: function (content, id) {
                sig.sendOverStream({
                    recipientId: id,
                    serviceId: _this.serviceId,
                    content: content instanceof Uint8Array ? content : _this.encode(content),
                });
            },
        };
    };
    Service.prototype.useAllStreams = function (wc, sig) {
        var wcStream = this.useWebChannelStream(wc);
        var sigStream = this.useSignalingStream(sig);
        return {
            message: merge(wcStream.message.pipe(map(function (_a) {
                var senderId = _a.senderId, msg = _a.msg;
                return ({ streamId: wcStream.id, senderId: senderId, msg: msg });
            })), sigStream.message.pipe(map(function (_a) {
                var senderId = _a.senderId, msg = _a.msg;
                return ({ streamId: sigStream.id, senderId: senderId, msg: msg });
            }))),
            sendOver: function (streamId, msg, id) {
                if (streamId === wcStream.id) {
                    wcStream.send(msg, id);
                }
                else {
                    sigStream.send(msg, id);
                }
            },
        };
    };
    Service.prototype.encode = function (msg) {
        return this.proto.encode(this.proto.create(msg)).finish();
    };
    Service.prototype.decode = function (bytes) {
        return this.proto.decode(bytes);
    };
    return Service;
}());
export { Service };
//# sourceMappingURL=Service.js.map