var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { userMessage as proto } from '../proto';
import { Service } from '../service/Service';
var MAX_USER_MSG_SIZE = 15000;
var MAX_MSG_ID_SIZE = 65535;
var textEncoder = new global.TextEncoder();
var textDecoder = new global.TextDecoder();
var UserMessage = (function (_super) {
    __extends(UserMessage, _super);
    function UserMessage() {
        var _this = _super.call(this, UserMessage.SERVICE_ID, proto.Message) || this;
        _this.buffers = new Map();
        return _this;
    }
    UserMessage.prototype.clean = function () {
        this.buffers.clear();
    };
    UserMessage.prototype.encodeUserMessage = function (data) {
        var _a = this.userDataToType(data), type = _a.type, bytes = _a.bytes;
        var msg = { length: bytes.length, type: type };
        if (bytes.length <= MAX_USER_MSG_SIZE) {
            msg.full = bytes;
            return [_super.prototype.encode.call(this, msg)];
        }
        else {
            msg.chunk = { id: Math.ceil(Math.random() * MAX_MSG_ID_SIZE) };
            var numberOfChunks = Math.ceil(bytes.length / MAX_USER_MSG_SIZE);
            var res = new Array(numberOfChunks);
            for (var i = 0; i < numberOfChunks; i++) {
                var length_1 = Math.min(MAX_USER_MSG_SIZE, bytes.length - MAX_USER_MSG_SIZE * i);
                var begin = MAX_USER_MSG_SIZE * i;
                var end = begin + length_1;
                msg.chunk.number = i;
                msg.chunk.content = new Uint8Array(bytes.slice(begin, end));
                res[i] = _super.prototype.encode.call(this, msg);
            }
            return res;
        }
    };
    UserMessage.prototype.decodeUserMessage = function (bytes, senderId) {
        var _a = _super.prototype.decode.call(this, bytes), length = _a.length, type = _a.type, contentType = _a.contentType, full = _a.full, chunk = _a.chunk;
        var result;
        switch (contentType) {
            case 'full': {
                result = full;
                break;
            }
            case 'chunk': {
                var buffer = this.getBuffer(senderId, chunk.id);
                if (buffer === undefined) {
                    buffer = new Buffer(length, chunk.content, chunk.nb);
                    this.setBuffer(senderId, chunk.id, buffer);
                    result = undefined;
                }
                else {
                    result = buffer.append(chunk.content, chunk.nb);
                }
                break;
            }
            default: {
                throw new Error('Unknown message integrity');
            }
        }
        if (result !== undefined) {
            switch (type) {
                case proto.Message.Type.U_INT_8_ARRAY:
                    return result;
                case proto.Message.Type.STRING:
                    return textDecoder.decode(result);
                default:
                    throw new Error('Unknown message type');
            }
        }
        return result;
    };
    UserMessage.prototype.userDataToType = function (data) {
        if (data instanceof Uint8Array) {
            return { type: proto.Message.Type.U_INT_8_ARRAY, bytes: data };
        }
        else if (typeof data === 'string') {
            return { type: proto.Message.Type.STRING, bytes: textEncoder.encode(data) };
        }
        else if (data instanceof String) {
            return { type: proto.Message.Type.STRING, bytes: textEncoder.encode('' + data) };
        }
        else {
            throw new Error('Message neigther a string type or a Uint8Array type');
        }
    };
    UserMessage.prototype.getBuffer = function (peerId, msgId) {
        var buffers = this.buffers.get(peerId);
        if (buffers !== undefined) {
            return buffers.get(msgId);
        }
        return undefined;
    };
    UserMessage.prototype.setBuffer = function (peerId, msgId, buffer) {
        var buffers = this.buffers.get(peerId);
        if (buffers === undefined) {
            buffers = new Map();
        }
        buffers.set(msgId, buffer);
        this.buffers.set(peerId, buffers);
    };
    UserMessage.SERVICE_ID = 743;
    return UserMessage;
}(Service));
export { UserMessage };
var Buffer = (function () {
    function Buffer(totalLength, data, chunkNb) {
        this.fullData = new Uint8Array(totalLength);
        this.currentLength = 0;
        this.append(data, chunkNb);
    }
    Buffer.prototype.append = function (data, chunkNb) {
        var i = chunkNb * MAX_USER_MSG_SIZE;
        this.currentLength += data.length;
        try {
            for (var data_1 = __values(data), data_1_1 = data_1.next(); !data_1_1.done; data_1_1 = data_1.next()) {
                var d = data_1_1.value;
                this.fullData[i++] = d;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (data_1_1 && !data_1_1.done && (_a = data_1.return)) _a.call(data_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return this.currentLength === this.fullData.length ? this.fullData : undefined;
        var e_1, _a;
    };
    return Buffer;
}());
//# sourceMappingURL=UserMessage.js.map